import itertools
import logging

from aiohttp import web

from feed_manager.engine.feed_engine import FeedEngine
from feed_manager.engine.watchlist_engine import WatchlistEngine
from feed_manager.web.route_helpers import make_response_json
from pscr_common.web.auth import AuthRouteTableDef

_logger = logging.getLogger(__name__)


def build_route_table(
    watchlist_engine: WatchlistEngine, feed_engine: FeedEngine
) -> AuthRouteTableDef:
    routes = AuthRouteTableDef()

    @routes.get("/v1/reports/{report_ids}")
    async def get_extended_reports(request: web.Request):
        # look out for log messages and eventually remove custom internal endpoint
        _logger.info("Hit get_extended_reports")

        report_ids = set(request.match_info["report_ids"].split(","))

        feed_reports = await feed_engine.get_extended_feed_reports(report_ids)

        not_feed_report_ids = report_ids - set(feed_reports.keys())

        custom_reports = await watchlist_engine.get_extended_reports(
            not_feed_report_ids
        )

        missing = not_feed_report_ids - set(custom_reports.keys())

        return make_response_json(
            {
                "reports": list(
                    itertools.chain(feed_reports.values(), custom_reports.values())
                ),
                "missing": list(missing),
            }
        )

    @routes.put("/v1/reports/{report_ids}/publish")
    async def publish_reports(request: web.Request):
        # look out for log messages and eventually remove custom internal endpoint
        _logger.info("Hit publish_reports")

        report_ids = set(request.match_info["report_ids"].split(","))

        await watchlist_engine.publisher.publish_report_notifications(
            report_ids=list(report_ids)
        )

        return make_response_json()

    return routes
